@@ -21,6 +21,7 @@
 (declare-function ai-code-backends-infra--terminal-send-string "ai-code-backends-infra" (string))
 (declare-function ai-code-backends-infra--terminal-send-backspace "ai-code-backends-infra" ())
 (declare-function ai-code--prompt-filepath-candidates "ai-code-prompt-mode" ())
+(declare-function ai-code--git-root "ai-code-file" (&optional dir))
 
 ;;;###autoload
 (defun ai-code-plain-read-string (prompt &optional initial-input candidate-list)
@@ -38,8 +39,14 @@ This function combines candidate-list with history for better completion."
                      nil nil initial-input
                      'ai-code-read-string-history)))
 
+(defvar ai-code--read-string-fn #'ai-code-plain-read-string
+  "Function used by `ai-code-read-string' to read user input.")
+
 ;;;###autoload
-(defalias 'ai-code-read-string #'ai-code-plain-read-string)
+(defun ai-code-read-string (prompt &optional initial-input candidate-list)
+  "Read a string from the user with PROMPT and optional INITIAL-INPUT.
+CANDIDATE-LIST provides additional completion options if provided."
+  (funcall ai-code--read-string-fn prompt initial-input candidate-list))
 
 (defun ai-code-helm-read-string-with-history (prompt history-file-name &optional initial-input candidate-list)
   "Read a string with Helm completion using specified history file.
@@ -101,8 +108,11 @@ CANDIDATE-LIST is an optional list of candidate strings to show before history."
   (ai-code-helm-read-string-with-history prompt "ai-code-helm-read-string-history.el" initial-input candidate-list))
 
 ;;;###autoload
-(if (featurep 'helm)
-    (defalias 'ai-code-read-string #'ai-code-helm-read-string))
+(when (featurep 'helm)
+  (setq ai-code--read-string-fn #'ai-code-helm-read-string))
+
+(with-eval-after-load 'helm
+  (setq ai-code--read-string-fn #'ai-code-helm-read-string))
 
 (defun ai-code--get-window-files ()
   "Get a list of unique file paths from all visible windows."
@@ -153,7 +163,8 @@ The current buffer's file is always first."
     (catch 'found
       (dolist (pattern patterns)
         (when (string-match pattern line)
-          (throw 'found (match-string 1 line))))
+          (let ((name (match-string 1 line)))
+            (throw 'found (replace-regexp-in-string ":+\\'" "" name)))))
       nil)))
 
 (defun ai-code--imenu-symbol-from-position (payload)
@@ -236,7 +247,7 @@ The current buffer's file is always first."
 (defun ai-code--hash-completion-target-file (&optional end-pos)
   "Return an absolute file path for @relative path ending at END-POS.
 END-POS defaults to the current '#' position."
-  (when-let ((git-root (magit-toplevel)))
+  (when-let ((git-root (ai-code--git-root)))
     (let* ((end (or end-pos (1- (point))))
            (start (save-excursion
                     (goto-char end)
@@ -249,10 +260,9 @@ END-POS defaults to the current '#' position."
                                (< start end)
                                (buffer-substring-no-properties start end))))
       (when relative-path
-        (let ((file (expand-file-name relative-path git-root))
-              (git-root-truename (file-truename git-root)))
+        (let ((file (expand-file-name relative-path git-root)))
           (when (and (file-regular-p file)
-                     (string-prefix-p git-root-truename (file-truename file)))
+                     (string-prefix-p git-root (file-truename file)))
             file))))))
 
 (defun ai-code--file-symbol-candidates (file)
@@ -280,7 +290,7 @@ END-POS defaults to the current '#' position."
              (ai-code--comment-context-p)
              (buffer-file-name)
              (not (minibufferp))
-             (magit-toplevel))
+             (ai-code--git-root))
     (let ((end (point))
           (start (save-excursion
                    (skip-chars-backward "A-Za-z0-9_./-")
@@ -318,7 +328,7 @@ END-POS defaults to the current '#' position."
              (fboundp 'ai-code-backends-infra--session-buffer-p)
              (ai-code-backends-infra--session-buffer-p (current-buffer))
              (not (minibufferp))
-             (magit-toplevel))
+             (ai-code--git-root))
     (pcase (char-before)
       (?@
        (let ((candidates (ai-code--prompt-filepath-candidates)))
@@ -342,7 +352,7 @@ END-POS defaults to the current '#' position."
               (fboundp 'ai-code-backends-infra--session-buffer-p)
               (ai-code-backends-infra--session-buffer-p (current-buffer))
               (not (minibufferp))
-              (magit-toplevel))))
+              (ai-code--git-root))))
     (ai-code-backends-infra--terminal-send-string "@")
     (when should-complete
       (let ((candidates (ai-code--prompt-filepath-candidates)))
@@ -362,7 +372,7 @@ END-POS defaults to the current '#' position."
                (fboundp 'ai-code-backends-infra--session-buffer-p)
                (ai-code-backends-infra--session-buffer-p (current-buffer))
                (not (minibufferp))
-               (magit-toplevel)))
+               (ai-code--git-root)))
          (file (and should-complete
                     (ai-code--hash-completion-target-file (point)))))
     (ai-code-backends-infra--terminal-send-string "#")
@@ -379,6 +389,7 @@ END-POS defaults to the current '#' position."
 (define-minor-mode ai-code-prompt-filepath-completion-mode
   "Toggle @ file completion in comments and AI sessions across all buffers."
   :global t
+  :group 'ai-code
   (setq ai-code-prompt-filepath-completion-enabled
         ai-code-prompt-filepath-completion-mode)
   (if ai-code-prompt-filepath-completion-mode
